@@ -247,6 +247,11 @@ def calculate_sharp_ratio(df_csv, col, risk_free_rate):
     net_profit = df.at[df.index[-1], col] - df.at[df.index[0], col]
 
     initial_capital = df.loc[df.index[0], col]
+
+    # To avoid power error below
+    if net_profit < 0 and abs(net_profit) > initial_capital:
+        net_profit = initial_capital * -1
+
     equity_value_pct_series = df[col].pct_change()
     equity_value_pct_series = equity_value_pct_series.dropna()
 
@@ -274,6 +279,7 @@ def resample_summary_to_daily(para_combination, folder=''):
     end_date = para_combination['end_date']
 
     sec_profile = para_combination['sec_profile']
+
     sectype = sec_profile['sectype']
     lot_size_dict = sec_profile['lot_size_dict']
     code = para_combination['code']
@@ -291,19 +297,34 @@ def resample_summary_to_daily(para_combination, folder=''):
     if file_format == 'parquet':
         df_backtest = pd.read_parquet(save_path) # Daraframe that may not be daily
         ### parquet specific, All backtest force to treart as summary, # ie filter action only
-        df_backtest = df_backtest.loc[df_backtest['action'] != ''].copy()
+        # df_backtest = df_backtest.loc[df_backtest['action'] != ''].copy()
+        # df_backtest = df_backtest.loc[df_backtest['action'].isnull() == False].copy()
     else:
         df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
         ### csv only specific, All backtest force to treart as summary, ie filter action only
-        df_backtest = df_backtest.loc[df_backtest['action'].isnull() == False].copy()
+        # df_backtest = df_backtest.loc[df_backtest['action'] != ''].copy()
+        # df_backtest = df_backtest.loc[df_backtest['action'].isnull() == False].copy()
+
+    df_backtest = df_backtest.loc[df_backtest['action'] != ''].copy()
+    df_backtest = df_backtest.loc[df_backtest['action'].isnull() == False].copy()
 
     df_backtest['date'] = pd.to_datetime(df_backtest['date'], format='%Y-%m-%d')
     df_backtest = df_backtest.loc[(df_backtest['date'] >= start_date) & (df_backtest['date'] <= end_date)]
     df_backtest = df_backtest.reset_index(drop=True)  # Reset Index
 
 
-    trade_pair = zip(*(iter(list(df_backtest.index)),) * 2)  # Group df_backtest by 2
-    initial_capital = df_backtest.iloc[-1].equity_value - df_backtest['realized_pnl'].sum()
+
+    df_daily = pd.DataFrame()  # Dataframe for Daily
+
+
+
+
+    # check if df_backtest 0 length:
+    if len(df_backtest) == 0:
+        initial_capital = sec_profile['initial_capital']
+    else:
+        initial_capital = df_backtest.iloc[-1].equity_value - df_backtest['realized_pnl'].sum()
+    # initial_capital = df_backtest.iloc[-1].equity_value - df_backtest['realized_pnl'].sum()
     last_realized_capital = initial_capital
 
 
@@ -323,7 +344,7 @@ def resample_summary_to_daily(para_combination, folder=''):
     df_csv = df_csv.reset_index(drop=False)
 
 
-    df_daily = pd.DataFrame()  # Dataframe for Daily
+
     df_daily['open'] = df_csv['open']
     df_daily['high'] = df_csv['high']
     df_daily['low'] = df_csv['low']
@@ -349,6 +370,9 @@ def resample_summary_to_daily(para_combination, folder=''):
 
 
 
+    trade_pair = zip(*(iter(list(df_backtest.index)),) * 2)  # Group df_backtest by 2
+
+
     for i, j in trade_pair:
         open_date = df_backtest.at[i, 'date']
         close_date = df_backtest.at[j, 'date']
@@ -475,6 +499,12 @@ def cal_performance(para_combination):
         df_backtest = df_backtest.reset_index(drop=True)  # Reset Index
         df_daily = df_backtest.copy()
 
+    df = df_daily.copy()
+
+    # Deter if equity_value unchange = no tade
+    equity_value_column = df['equity_value'].to_numpy()
+    no_trade = (equity_value_column[0] == equity_value_column).all()
+
     result_dict = {}
 
     # Determine years at the beginning
@@ -483,92 +513,130 @@ def cal_performance(para_combination):
     year_list = list(range(start_date_year, end_date_year + 1))
     for y in year_list: result_dict[str(y)] = []
 
-    # df = pd.read_csv(save_path)
-    df = df_daily.copy()
-    df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')
 
-    # Strategy Performance
-    net_profit, holding_period_day, return_on_capital, annualized_return, annualized_std, annualized_sr = calculate_sharp_ratio(df, 'equity_value', risk_free_rate)
-    mdd_pct, mdd_dollar = calculate_mdd(df, 'equity_value')
-    mdd_pct = mdd_pct * -100
-    mdd_dollar = mdd_dollar * -1
+    # if length of backtest is zero, no trade, no performance
+    if no_trade:
+        return_on_capital = 0
+        result_dict['holding_period_day'] = 0
+        result_dict['total_commission'] = 0
+        result_dict['net_profit'] = 0
+        result_dict['return_on_capital'] = 0
+        result_dict['annualized_return'] = 0
+        result_dict['annualized_std'] = 0
+        result_dict['annualized_sr'] = 0
+        result_dict['mdd_dollar'] = 0
+        result_dict['mdd_pct'] = 0
+        result_dict['num_of_trade'] = 0
+        result_dict['win_rate'] = 0
+        result_dict['loss_rate'] = 0
+        result_dict['net_profit_to_mdd'] = np.inf
+        result_dict['cov'] = 0
 
-    # Win Rate (need to use df_backtest directly)
 
-    save_path = generate_filepath(para_combination=para_combination, file_format=file_format)
-    if file_format == 'parquet':
-        df_backtest = pd.read_parquet(save_path)  # Daraframe that may not be daily
+        # Win rate by year
+        for year in year_list:
+            result_dict[str(year)] = 0
+            result_dict[f'{year}_win_rate'] = '--'
+            result_dict[f'{year}_return'] = 0
+
     else:
-        df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
+        df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')
+        df['year'] = pd.DatetimeIndex(df['date']).year
 
-    win_rate_dict = calculate_win_rate(df_backtest)
-    num_of_trade, num_of_loss, num_of_win, win_rate, loss_rate = win_rate_dict['Overall']
-    # print('net_profit',net_profit)
+        # Strategy Performance
+        net_profit, holding_period_day, return_on_capital, annualized_return, annualized_std, annualized_sr = calculate_sharp_ratio(df, 'equity_value', risk_free_rate)
+        mdd_pct, mdd_dollar = calculate_mdd(df, 'equity_value')
+        mdd_pct = mdd_pct * -100
+        mdd_dollar = mdd_dollar * -1
+
+        # Win Rate (need to use df_backtest directly)
+
+        save_path = generate_filepath(para_combination=para_combination, file_format=file_format)
+        if file_format == 'parquet':
+            df_backtest = pd.read_parquet(save_path)  # Daraframe that may not be daily
+        else:
+            df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
+
+        win_rate_dict = calculate_win_rate(df_backtest)
+        num_of_trade, num_of_loss, num_of_win, win_rate, loss_rate = win_rate_dict['Overall']
+        # print('net_profit',net_profit)
+
+        total_commission = df['commission'].sum()
+
+        result_dict['holding_period_day'] = holding_period_day
+        result_dict['total_commission'] = total_commission
+
+        result_dict['net_profit'] = net_profit
+        result_dict['return_on_capital'] = return_on_capital
+        result_dict['annualized_return'] = annualized_return
+        result_dict['annualized_std'] = annualized_std
+        result_dict['annualized_sr'] = annualized_sr
+        result_dict['mdd_dollar'] = mdd_dollar
+        result_dict['mdd_pct'] = mdd_pct
+        result_dict['num_of_trade'] = num_of_trade
+        result_dict['win_rate'] = win_rate
+        result_dict['loss_rate'] = loss_rate
+
+        if mdd_dollar == 0:
+            result_dict['net_profit_to_mdd'] = np.inf
+        else:
+            result_dict['net_profit_to_mdd'] = net_profit / mdd_dollar
+
+
+        # Cov
+        df3 = df[df['action'] == 'open']
+        df3 = df3.set_index('date')
+        signal_year_count = df3.groupby(lambda x: x.year).size()
+
+        signal_year_std = np.std(signal_year_count)
+        signal_year_mean = np.mean(signal_year_count)
+        cov = round(signal_year_std / signal_year_mean, 3)
+
+        result_dict['cov'] = cov
+
+        # Win rate by year
+        for year in year_list:
+            try:
+                result_dict[str(year)] = win_rate_dict[year][0]
+                result_dict[f'{year}_win_rate'] = win_rate_dict[year][3]
+
+            except Exception as e:
+                # print(e)
+                result_dict[str(year)] = 0
+                result_dict[f'{year}_win_rate'] = '--'
+
+
+        # Performance by year
+        first_equity_value = 0
+        for year in year_list:
+            if first_equity_value == 0:
+                first_equity_value = df.loc[df['year'] == year].iloc[0].equity_value
+            last_equity_value = df.loc[df['year'] == year].iloc[-1].equity_value
+            yearly_return = (last_equity_value - first_equity_value) / first_equity_value
+            result_dict[f'{year}_return'] = int(yearly_return*100)
+            first_equity_value = last_equity_value
+
+
+
+    result_dict['risk_free_rate'] = risk_free_rate
 
     # BaH Performance
     bah_net_return, holding_period_day, bah_return, \
-    bah_annualized_return, bah_annualized_std, bah_annualized_sr = calculate_sharp_ratio(df, 'close', risk_free_rate)
+    bah_annualized_return, bah_annualized_std, bah_annualized_sr = calculate_sharp_ratio(df, 'close',
+                                                                                         risk_free_rate)
     initial_capital = df.loc[df.index[0], 'equity_value']
     df['bah_equity_curve'] = df['close'] * initial_capital // df.loc[df.index[0], 'close']
     bah_mdd_pct, bah_mdd_dollar = calculate_mdd(df, 'bah_equity_curve')
     bah_mdd_pct = bah_mdd_pct * -100
     bah_mdd_dollar = bah_mdd_dollar * -1
 
-    total_commission = df['commission'].sum()
-
-    result_dict['holding_period_day'] = holding_period_day
-    result_dict['total_commission'] = total_commission
-
-    result_dict['net_profit'] = net_profit
-    result_dict['return_on_capital'] = return_on_capital
-    result_dict['annualized_return'] = annualized_return
-    result_dict['annualized_std'] = annualized_std
-    result_dict['annualized_sr'] = annualized_sr
-    result_dict['mdd_dollar'] = mdd_dollar
-    result_dict['mdd_pct'] = mdd_pct
-    result_dict['num_of_trade'] = num_of_trade
-    result_dict['win_rate'] = win_rate
-    result_dict['loss_rate'] = loss_rate
-    if mdd_dollar == 0:
-        result_dict['net_profit_to_mdd'] = np.inf
-    else:
-        result_dict['net_profit_to_mdd'] = net_profit / mdd_dollar
-
     result_dict['bah_return'] = bah_return
     result_dict['bah_annualized_return'] = bah_annualized_return
     result_dict['bah_annualized_std'] = bah_annualized_std
     result_dict['bah_annualized_sr'] = bah_annualized_sr
     result_dict['bah_mdd_dollar'] = bah_mdd_dollar
     result_dict['bah_mdd_pct'] = bah_mdd_pct
-
-    # Cov
-    df3 = df[df['action'] == 'open']
-    df3 = df3.set_index('date')
-    signal_year_count = df3.groupby(lambda x: x.year).size()
-
-    signal_year_std = np.std(signal_year_count)
-    signal_year_mean = np.mean(signal_year_count)
-    cov = round(signal_year_std / signal_year_mean, 3)
-
-    result_dict['cov'] = cov
-
-    # Win rate by year
-    for year in year_list:
-        try:
-            result_dict[str(year)] = win_rate_dict[year][0]
-            result_dict[f'{year}_win_rate'] = win_rate_dict[year][3]
-
-        except Exception as e:
-            # print(e)
-            result_dict[str(year)] = 0
-            result_dict[f'{year}_win_rate'] = '--'
-
-
-    # Performance by year
-
-
     result_dict['return_to_bah'] = return_on_capital - bah_return
-    result_dict['risk_free_rate'] = risk_free_rate
 
 
 
