@@ -332,6 +332,23 @@ class Aggregate:
             end_date_year = df_all_curves.loc[df_all_curves.index[-1], 'date'].year
             year_list = list(range(start_date_year, end_date_year + 1))
 
+
+            # Performance by year
+            df_year = pd.DataFrame()
+            df_year['equity_value'] = df_all_curves['Aggregate_Equity'].copy()
+            df_year['date'] = df_agg.index.copy()
+            df_year['date'] = pd.to_datetime(df_year['date'], format='%Y-%m-%d')
+            df_year['year'] = pd.DatetimeIndex(df_year['date']).year
+
+            first_equity_value = 0
+            for year in year_list:
+                if first_equity_value == 0:
+                    first_equity_value = df_year.loc[df_year['year'] == year].iloc[0].equity_value
+                last_equity_value = df_year.loc[df_year['year'] == year].iloc[-1].equity_value
+                yearly_return = (last_equity_value - first_equity_value) / first_equity_value
+                total_dict[f'{year}_return'] = yearly_return
+                first_equity_value = last_equity_value
+
             aggregate_performance = components.aggregate_performance(total_dict, year_list, risk_free_rate)
 
 
